//################################################################################
//# Project: Fire Curtain System
//# Company: [Company Name]
//# Author: Supriya Kore
//# Date: 20-July-2023
//# Description: [Macros Declaration File]
//#
//# [Additional project or code-specific information]
//#
//# [Company Name]
//# [Company Address]
//# [Company Website]
//# [Contact Information]
//################################################################################
#ifndef FIRE_CURTAIN_H
#define FIRE_CURTAIN_H

/*********************************Header Files***********************************/
#include <stdint.h>
#include "Motor.h"
#include "Buzzer.h"
#include "Scan.h"
/***********************Functions Prototype Declaration***************************/

bool IRAM_ATTR TimerHandler1(void * timerNo);
void irqGenericTimer();
void MainsStatus();
void inputScan();
void Batt_Mon();

/*****************************Enum Declaration***********************************/
typedef enum State {
  STATE_INIT,
  STATE_READY,
  STATE_MOTOR_CONTROL,
  STATE_MOTOR_STOP,
  STATE_ERROR
} State_t;

/******************************Timers**********************************************/
//struct debounceTmr{
  uint8_t Fire_tmr = 0;
  uint8_t Beam_tmr = 0;
  uint8_t KeyUp_tmr = 0;
  uint8_t KeyDown_tmr = 0;
  uint8_t Override_tmr = 0;
//} debounceTimer;

//struct timers{
  uint8_t FireMtrStop_tmr = 0;
  uint8_t BeamMtrUp_tmr = 0;
  uint8_t BattLowMtrStop_tmr = 0;
  uint8_t BeamWait_tmr = 0;
  uint8_t MtrStop_tmr = 0;
//}Timers;


uint8_t OverrideRelFire_tmr = 0;
uint8_t KeyUpRelFire_tmr = 0;
/*****************************************************************************************/
unsigned char FireDeb_Flag = 0;
unsigned char BeamDeb_Flag = 0;
unsigned char KeyDownDeb_Flag = 0;
unsigned char KeyUpDeb_Flag = 0;
unsigned char OverrideDeb_Flag = 0;
  

unsigned char testFlag = 0;
bool Fire_Flag = false;
bool Beam_M1_Flag = false;
bool boolKeyUpFlag = false;
bool boolKeyDownFlag = false;
bool boolOverrideKeyFlag = false;
/******************************PIN Declaration************************************/
/******************************OUTPUT PINS****************************************/
#define PFC                   0     //PFC is connected to GPIO 0
#define SIREN                 1 //37     //Siren is connected to GPIO 23 //changed pin1 to pin23
#define BUZZER                2 //38     //Buzzer is connected to GPIO 2
#define M1_UP                 5      //Motor_1_Up is connected to GPIO 5
#define M1_DOWN               6 //26     //Motor_1_Down is connected to GPIO 26 //changed pin6 to pin26
#define M2_UP                 7 //27     //Motor_2_Up is connected to GPIO 7 //changed pin 7 to pin27
#define M2_DOWN               8 //33     //Motor_2_Down is connected to GPIO 8 //changed pin8 to pin33

/*******************************INPUT PINS*****************************************/
#define BEAM_DETECTOR_M1      13 //36 //23     //Beam_Detector_M1 is connected to GPIO 13 //STOP heare
#define SMOKE_DETECTOR        37 //32     //Smoke_Detector is connected to GPIO 16 //changed pin 37 to pin 16
#define FIRE_PANEL_INPUT      38 //0      //Fire_Panel_Input is connected to GPIO 17 //changed pin 38 to pin 17
#define KEY_SWITCH_UP         45 //21     //Key_Switch_M1 is connected to GPIO 21 //changed pin 45 to pin 21
#define KEY_SWITCH_DOWN       46 //17     //Key_Switch_M2 is connected to GPIO 22 //changed pin 46 to pin 22
#define OVERRIDE_SWITCH       21

/****************************Key PINS Macros****************************************/
#define MENU_KEY              26
#define REST_RIGHT_SHIFT_KEY  27
#define ENTER_KEY             28
#define DOWN_KEY              12      //Limit_M2_DOWN
#define UP_KEY                11     //Limit_M2_UP
#define LEFT_SHIFT_KEY        14      //Beam_M2 

#define BATT_MONITOR          3
//#define MAINS                 47

#define ADC_RESOLUTION        12     // Initialize Adc Resolution as 12 bits.
#define BAUD_RATE             115200 // Initialize Baudrate as 115200.

/********************************LED Pins defination*******************************/

#define SystemOn    P0
#define FireAlarm   P1
#define ManOverride P2
#define MotorON     P3
#define CommFault   P4
#define BatteryFault P5
#define MainsFail   P6
#define Wifi        P7

/**************************Keypad pin Macros****************************************/



/***********************************************************************************/
volatile bool boolSmokeInterrupt = false;                   //Flag to indicate whether smoke interrupt arrived or not
volatile bool boolFirePanelInterrupt = false;               //Flag to indicate whether Fire interrupt arrived or Not

volatile bool boolKeySwitchDownPressed;                     //Flag to indicate if Key Down is pressed or released
volatile bool boolKeySwitchUpPressed;                       //Flag to indicate if Key Up is pressed or released

volatile bool boolKeyDownInterrupt = false;                 //Flag to indicate whether Key down interrupt is arrived or not
volatile bool boolKeyUpInterrupt = false;                   //Flag to indicate whether Key up interrupt is arrived or not

volatile bool boolBeamDetectorM1Pressed = false;            //Flag to indicate whether Beam Detector M2 pressed or released

volatile bool boolBeamM1Interrupt = false;                  //Flag to indicate whether Beam_M1 interrupt is arrived or not

volatile bool boolOverrideSwitchInterrupt = false;
volatile bool boolOverrideSwitchPressed = false;

volatile bool boolBeamFireWait = false;
volatile bool boolBeamTimerOver = false;
volatile bool boolBattryLowTimerOver = false;
volatile bool boolSmokeTimerOver = false;
volatile bool boolFireTimerOver = false;
volatile bool boolBeamPresentWait = false;
volatile bool boolBeamAbsSmkFireWait = false;
volatile bool boolMotorStop = false;
bool boolMotorStop1 = false;
bool boolMotorStop2 = false;
/*************************************************************************************/
volatile bool variable1 = false;

volatile bool boolBeamSmokeWait = false;

volatile bool boolBatteryLowMotorFlag = false;
volatile bool boolBatteryLowFlag = false;

/*******************************************************************************/





/*******************************************************************************/
volatile bool boolMainsFailFlag = false;
volatile bool boolSystemNormalFlag = true;
//volatile bool boolvariable1 = false;
//volatile bool boolvariable2 = false;
/*******************************************************************************/

uint8_t count50ms = 0;
uint8_t testTimer = 0;
/******************************I2C Initialization*******************************/
#define I2C_SDA  35
#define I2C_SCL  36
int lcdColumns = 16;
int lcdRows = 2;
LiquidCrystal_I2C lcd(0x27, lcdColumns, lcdRows);
PCF8574 pcf8574(0x20);
/************************************Timer*************************************/
Ticker BeamTimer;
Ticker BeamWaitTimer;
Ticker SmokeFireBeamWaitTimer;
Ticker OverrideDebounceTimer;
#define TIMER1_INTERVAL_MS        50                //50ms timer
// Init ESP32 timer 1
ESP32Timer ITimer1(1);
// Init ESP32_ISR_Timer
ESP32_ISR_Timer ISR_Timer;




void timerBatteryLow()
{
  if(boolBatteryLowMotorFlag == false)
  {
    boolBattryLowTimerOver = true;
    boolBatteryLowMotorFlag = true;
  }
}

#endif
